@@ -1,16 +1,19 @@
 import collections
 from http import cookiejar
-from typing import List, Tuple, Dict, Optional  # noqa
+from typing import Optional
 
-from mitmproxy import http, flowfilter, ctx, exceptions
+from mitmproxy import ctx
+from mitmproxy import exceptions
+from mitmproxy import flowfilter
+from mitmproxy import http
 from mitmproxy.net.http import cookies
 
-TOrigin = Tuple[str, int, str]
+TOrigin = tuple[str, int, str]
 
 
-def ckey(attrs: Dict[str, str], f: http.HTTPFlow) -> TOrigin:
+def ckey(attrs: dict[str, str], f: http.HTTPFlow) -> TOrigin:
     """
-        Returns a (domain, port, path) tuple.
+    Returns a (domain, port, path) tuple.
     """
     domain = f.request.host
     path = "/"
@@ -30,29 +33,32 @@ def domain_match(a: str, b: str) -> bool:
 
 
 class StickyCookie:
-    def __init__(self):
-        self.jar = collections.defaultdict(dict)  # type: Dict[TOrigin, Dict[str, str]]
-        self.flt = None  # type: Optional[flowfilter.TFilter]
+    def __init__(self) -> None:
+        self.jar: collections.defaultdict[
+            TOrigin, dict[str, str]
+        ] = collections.defaultdict(dict)
+        self.flt: flowfilter.TFilter | None = None
 
     def load(self, loader):
         loader.add_option(
-            "stickycookie", Optional[str], None,
-            "Set sticky cookie filter. Matched against requests."
+            "stickycookie",
+            Optional[str],
+            None,
+            "Set sticky cookie filter. Matched against requests.",
         )
 
     def configure(self, updated):
         if "stickycookie" in updated:
             if ctx.options.stickycookie:
-                flt = flowfilter.parse(ctx.options.stickycookie)
-                if not flt:
-                    raise exceptions.OptionsError(
-                        "stickycookie: invalid filter expression: %s" % ctx.options.stickycookie
-                    )
-                self.flt = flt
+                try:
+                    self.flt = flowfilter.parse(ctx.options.stickycookie)
+                except ValueError as e:
+                    raise exceptions.OptionsError(str(e)) from e
             else:
                 self.flt = None
 
     def response(self, flow: http.HTTPFlow):
+        assert flow.response
         if self.flt:
             for name, (value, attrs) in flow.response.cookies.items(multi=True):
                 # FIXME: We now know that Cookie.py screws up some cookies with
@@ -73,17 +79,19 @@ class StickyCookie:
 
     def request(self, flow: http.HTTPFlow):
         if self.flt:
-            cookie_list = []  # type: List[Tuple[str,str]]
+            cookie_list: list[tuple[str, str]] = []
             if flowfilter.match(self.flt, flow):
                 for (domain, port, path), c in self.jar.items():
                     match = [
                         domain_match(flow.request.host, domain),
                         flow.request.port == port,
-                        flow.request.path.startswith(path)
+                        flow.request.path.startswith(path),
                     ]
                     if all(match):
                         cookie_list.extend(c.items())
             if cookie_list:
                 # FIXME: we need to formalise this...
                 flow.metadata["stickycookie"] = True
-                flow.request.headers["cookie"] = cookies.format_cookie_header(cookie_list)
+                flow.request.headers["cookie"] = cookies.format_cookie_header(
+                    cookie_list
+                )
