@@ -1,8 +1,6 @@
-from typing import Optional
-
+from . import base
 from mitmproxy.coretypes import multidict
 from mitmproxy.net.http import multipart
-from . import base
 
 
 class ViewMultipart(base.View):
@@ -13,12 +11,14 @@ class ViewMultipart(base.View):
         yield [("highlight", "Form data:\n")]
         yield from base.format_dict(multidict.MultiDict(v))
 
-    def __call__(self, data: bytes, content_type: Optional[str] = None, **metadata):
+    def __call__(self, data: bytes, content_type: str | None = None, **metadata):
         if content_type is None:
             return
-        v = multipart.decode(content_type, data)
+        v = multipart.decode_multipart(content_type, data)
         if v:
             return "Multipart form", self._format(v)
 
-    def render_priority(self, data: bytes, *, content_type: Optional[str] = None, **metadata) -> float:
-        return float(content_type == "multipart/form-data")
+    def render_priority(
+        self, data: bytes, *, content_type: str | None = None, **metadata
+    ) -> float:
+        return float(bool(data) and content_type == "multipart/form-data")
