@@ -2,7 +2,9 @@ import asyncio
 import logging
 
 from . import ctx as mitmproxy_ctx
+from .addons import termlog
 from .proxy.mode_specs import ReverseMode
+from .utils import asyncio_utils
 from mitmproxy import addonmanager
 from mitmproxy import command
 from mitmproxy import eventsequence
@@ -20,16 +22,22 @@ class Master:
     """
 
     event_loop: asyncio.AbstractEventLoop
+    _termlog_addon: termlog.TermLog | None = None
 
     def __init__(
         self,
         opts: options.Options,
         event_loop: asyncio.AbstractEventLoop | None = None,
+        with_termlog: bool = False,
     ):
         self.options: options.Options = opts or options.Options()
         self.commands = command.CommandManager(self)
         self.addons = addonmanager.AddonManager(self)
 
+        if with_termlog:
+            self._termlog_addon = termlog.TermLog()
+            self.addons.add(self._termlog_addon)
+
         self.log = log.Log(self)  # deprecated, do not use.
         self._legacy_log_events = log.LegacyLogEvents(self)
         self._legacy_log_events.install()
@@ -44,9 +52,10 @@ class Master:
         mitmproxy_ctx.options = self.options
 
     async def run(self) -> None:
-        old_handler = self.event_loop.get_exception_handler()
-        self.event_loop.set_exception_handler(self._asyncio_exception_handler)
-        try:
+        with (
+            asyncio_utils.install_exception_handler(self._asyncio_exception_handler),
+            asyncio_utils.set_eager_task_factory(),
+        ):
             self.should_exit.clear()
 
             if ec := self.addons.get("errorcheck"):
@@ -60,17 +69,15 @@ class Master:
                     ],
                     return_when=asyncio.FIRST_COMPLETED,
                 )
+            await self.running()
             if ec := self.addons.get("errorcheck"):
                 await ec.shutdown_if_errored()
                 ec.finish()
-            await self.running()
             try:
                 await self.should_exit.wait()
             finally:
                 # .wait might be cancelled (e.g. by sys.exit)
                 await self.done()
-        finally:
-            self.event_loop.set_exception_handler(old_handler)
 
     def shutdown(self):
         """
@@ -85,6 +92,8 @@ class Master:
     async def done(self) -> None:
         await self.addons.trigger_event(hooks.DoneHook())
         self._legacy_log_events.uninstall()
+        if self._termlog_addon is not None:
+            self._termlog_addon.uninstall()
 
     def _asyncio_exception_handler(self, loop, context) -> None:
         try:
