@@ -1,16 +1,18 @@
 import re
 import time
-from typing import List, Tuple, Iterable, Optional
+from collections.abc import Iterable
 
-from mitmproxy.http import Request, Headers, Response
+from mitmproxy.http import Headers
+from mitmproxy.http import Request
+from mitmproxy.http import Response
 from mitmproxy.net.http import url
 
 
 def get_header_tokens(headers, key):
     """
-        Retrieve all tokens for a header key. A number of different headers
-        follow a pattern where each header line can containe comma-separated
-        tokens, and headers can be set multiple times.
+    Retrieve all tokens for a header key. A number of different headers
+    follow a pattern where each header line can containe comma-separated
+    tokens, and headers can be set multiple times.
     """
     if key not in headers:
         return []
@@ -20,10 +22,10 @@ def get_header_tokens(headers, key):
 
 def connection_close(http_version, headers):
     """
-        Checks the message to see if the client connection should be closed
-        according to RFC 2616 Section 8.1.
-        If we don't have a Connection header, HTTP 1.1 connections are assumed
-        to be persistent.
+    Checks the message to see if the client connection should be closed
+    according to RFC 2616 Section 8.1.
+    If we don't have a Connection header, HTTP 1.1 connections are assumed
+    to be persistent.
     """
     if "connection" in headers:
         tokens = get_header_tokens(headers, "connection")
@@ -33,24 +35,59 @@ def connection_close(http_version, headers):
             return False
 
     return http_version not in (
-        "HTTP/1.1", b"HTTP/1.1",
-        "HTTP/2.0", b"HTTP/2.0",
+        "HTTP/1.1",
+        b"HTTP/1.1",
+        "HTTP/2.0",
+        b"HTTP/2.0",
     )
 
 
-def expected_http_body_size(
-        request: Request,
-        response: Optional[Response] = None
-) -> Optional[int]:
-    """
-        Returns:
-            The expected body length:
-            - a positive integer, if the size is known in advance
-            - None, if the size in unknown in advance (chunked encoding)
-            - -1, if all data should be read until end of stream.
+# https://datatracker.ietf.org/doc/html/rfc7230#section-3.2: Header fields are tokens.
+# "!" / "#" / "$" / "%" / "&" / "'" / "*" / "+" / "-" / "." /  "^" / "_" / "`" / "|" / "~" / DIGIT / ALPHA
+_valid_header_name = re.compile(rb"^[!#$%&'*+\-.^_`|~0-9a-zA-Z]+$")
 
-        Raises:
-            ValueError, if the content length header is invalid
+
+def validate_headers(headers: Headers) -> None:
+    """
+    Validate headers to avoid request smuggling attacks. Raises a ValueError if they are malformed.
+    """
+
+    te_found = False
+    cl_found = False
+
+    for name, value in headers.fields:
+        if not _valid_header_name.match(name):
+            raise ValueError(
+                f"Received an invalid header name: {name!r}. Invalid header names may introduce "
+                f"request smuggling vulnerabilities. Disable the validate_inbound_headers option "
+                f"to skip this security check."
+            )
+
+        name_lower = name.lower()
+        te_found = te_found or name_lower == b"transfer-encoding"
+        cl_found = cl_found or name_lower == b"content-length"
+
+    if te_found and cl_found:
+        raise ValueError(
+            "Received both a Transfer-Encoding and a Content-Length header, "
+            "refusing as recommended in RFC 7230 Section 3.3.3. "
+            "See https://github.com/mitmproxy/mitmproxy/issues/4799 for details. "
+            "Disable the validate_inbound_headers option to skip this security check."
+        )
+
+
+def expected_http_body_size(
+    request: Request, response: Response | None = None
+) -> int | None:
+    """
+    Returns:
+        The expected body length:
+        - a positive integer, if the size is known in advance
+        - None, if the size in unknown in advance (chunked encoding)
+        - -1, if all data should be read until end of stream.
+
+    Raises:
+        ValueError, if the content length header is invalid
     """
     # Determine response size according to http://tools.ietf.org/html/rfc7230#section-3.3, which is inlined below.
     if not response:
@@ -101,10 +138,8 @@ def expected_http_body_size(
     #        a message downstream.
     #
     if "transfer-encoding" in headers:
-        if "content-length" in headers:
-            raise ValueError("Received both a Transfer-Encoding and a Content-Length header, "
-                             "refusing as recommended in RFC 7230 Section 3.3.3. "
-                             "See https://github.com/mitmproxy/mitmproxy/issues/4799 for details.")
+        # we should make sure that there isn't also a content-length header.
+        # this is already handled in validate_headers.
 
         te: str = headers["transfer-encoding"]
         if not te.isascii():
@@ -128,9 +163,13 @@ def expected_http_body_size(
             if response:
                 return -1
             else:
-                raise ValueError(f"Invalid request transfer encoding, message body cannot be determined reliably.")
+                raise ValueError(
+                    f"Invalid request transfer encoding, message body cannot be determined reliably."
+                )
         else:
-            raise ValueError(f"Unknown transfer encoding: {headers['transfer-encoding']!r}")
+            raise ValueError(
+                f"Unknown transfer encoding: {headers['transfer-encoding']!r}"
+            )
 
     #    4.  If a message is received without Transfer-Encoding and with
     #        either multiple Content-Length header fields having differing
@@ -155,10 +194,13 @@ def expected_http_body_size(
         sizes = headers.get_all("content-length")
         different_content_length_headers = any(x != sizes[0] for x in sizes)
         if different_content_length_headers:
-            raise ValueError("Conflicting Content Length Headers")
-        size = int(sizes[0])
+            raise ValueError(f"Conflicting Content-Length headers: {sizes!r}")
+        try:
+            size = int(sizes[0])
+        except ValueError:
+            raise ValueError(f"Invalid Content-Length header: {sizes[0]!r}")
         if size < 0:
-            raise ValueError("Negative Content Length")
+            raise ValueError(f"Negative Content-Length header: {sizes[0]!r}")
         return size
 
     #    6.  If this is a request message and none of the above are true, then
@@ -174,14 +216,16 @@ def expected_http_body_size(
 
 
 def raise_if_http_version_unknown(http_version: bytes) -> None:
-    if not re.match(br"^HTTP/\d\.\d$", http_version):
+    if not re.match(rb"^HTTP/\d\.\d$", http_version):
         raise ValueError(f"Unknown HTTP version: {http_version!r}")
 
 
-def _read_request_line(line: bytes) -> Tuple[str, int, bytes, bytes, bytes, bytes, bytes]:
+def _read_request_line(
+    line: bytes,
+) -> tuple[str, int, bytes, bytes, bytes, bytes, bytes]:
     try:
         method, target, http_version = line.split()
-        port: Optional[int]
+        port: int | None
 
         if target == b"*" or target.startswith(b"/"):
             scheme, authority, path = b"", b"", target
@@ -209,7 +253,7 @@ def _read_request_line(line: bytes) -> Tuple[str, int, bytes, bytes, bytes, byte
     return host, port, method, scheme, authority, path, http_version
 
 
-def _read_response_line(line: bytes) -> Tuple[bytes, int, bytes]:
+def _read_response_line(line: bytes) -> tuple[bytes, int, bytes]:
     try:
         parts = line.split(None, 2)
         if len(parts) == 2:  # handle missing message gracefully
@@ -226,22 +270,22 @@ def _read_response_line(line: bytes) -> Tuple[bytes, int, bytes]:
 
 def _read_headers(lines: Iterable[bytes]) -> Headers:
     """
-        Read a set of headers.
-        Stop once a blank line is reached.
+    Read a set of headers.
+    Stop once a blank line is reached.
 
-        Returns:
-            A headers object
+    Returns:
+        A headers object
 
-        Raises:
-            exceptions.HttpSyntaxException
+    Raises:
+        exceptions.HttpSyntaxException
     """
-    ret: List[Tuple[bytes, bytes]] = []
+    ret: list[tuple[bytes, bytes]] = []
     for line in lines:
         if line[0] in b" \t":
             if not ret:
                 raise ValueError("Invalid headers")
             # continued header
-            ret[-1] = (ret[-1][0], ret[-1][1] + b'\r\n ' + line.strip())
+            ret[-1] = (ret[-1][0], ret[-1][1] + b"\r\n " + line.strip())
         else:
             try:
                 name, value = line.split(b":", 1)
@@ -254,7 +298,7 @@ def _read_headers(lines: Iterable[bytes]) -> Headers:
     return Headers(ret)
 
 
-def read_request_head(lines: List[bytes]) -> Request:
+def read_request_head(lines: list[bytes]) -> Request:
     """
     Parse an HTTP request head (request line + headers) from an iterable of lines
 
@@ -267,7 +311,9 @@ def read_request_head(lines: List[bytes]) -> Request:
     Raises:
         ValueError: The input is malformed.
     """
-    host, port, method, scheme, authority, path, http_version = _read_request_line(lines[0])
+    host, port, method, scheme, authority, path, http_version = _read_request_line(
+        lines[0]
+    )
     headers = _read_headers(lines[1:])
 
     return Request(
@@ -282,11 +328,11 @@ def read_request_head(lines: List[bytes]) -> Request:
         content=None,
         trailers=None,
         timestamp_start=time.time(),
-        timestamp_end=None
+        timestamp_end=None,
     )
 
 
-def read_response_head(lines: List[bytes]) -> Response:
+def read_response_head(lines: list[bytes]) -> Response:
     """
     Parse an HTTP response head (response line + headers) from an iterable of lines
 
