@@ -90,11 +90,19 @@ class Options(optmanager.OptManager):
             """,
         )
         self.add_option("allow_hosts", Sequence[str], [], "Opposite of --ignore-hosts.")
-        self.add_option("listen_host", str, "",
-                        "Address to bind proxy server(s) to (may be overridden for individual modes, see `mode`).")
-        self.add_option("listen_port", Optional[int], None,
-                        "Port to bind proxy server(s) to (may be overridden for individual modes, see `mode`). "
-                        "By default, the port is mode-specific. The default regular HTTP proxy spawns on port 8080.")
+        self.add_option(
+            "listen_host",
+            str,
+            "",
+            "Address to bind proxy server(s) to (may be overridden for individual modes, see `mode`).",
+        )
+        self.add_option(
+            "listen_port",
+            Optional[int],
+            None,
+            "Port to bind proxy server(s) to (may be overridden for individual modes, see `mode`). "
+            "By default, the port is mode-specific. The default regular HTTP proxy spawns on port 8080.",
+        )
         self.add_option(
             "mode",
             Sequence[str],
@@ -103,8 +111,9 @@ class Options(optmanager.OptManager):
             The proxy server type(s) to spawn. Can be passed multiple times.
 
             Mitmproxy supports "regular" (HTTP), "transparent", "socks5", "reverse:SPEC",
-            and "upstream:SPEC" proxy servers. For reverse and upstream proxy modes, SPEC
-            is host specification in the form of "http[s]://host[:port]".
+            "upstream:SPEC", and "wireguard[:PATH]" proxy servers. For reverse and upstream proxy modes, SPEC
+            is host specification in the form of "http[s]://host[:port]". For WireGuard mode, PATH may point to
+            a file containing key material. If no such file exists, it will be created on startup.
 
             You may append `@listen_port` or `@listen_host:listen_port` to override `listen_host` or `listen_port` for
             a specific proxy mode. Features such as client playback will use the first mode to determine
@@ -122,7 +131,7 @@ class Options(optmanager.OptManager):
             "http2",
             bool,
             True,
-            "Enable/disable HTTP/2 support. " "HTTP/2 support is enabled by default.",
+            "Enable/disable HTTP/2 support. HTTP/2 support is enabled by default.",
         )
         self.add_option(
             "http2_ping_keepalive",
@@ -134,6 +143,12 @@ class Options(optmanager.OptManager):
             Set to 0 to disable this feature.
             """,
         )
+        self.add_option(
+            "http3",
+            bool,
+            True,
+            "Enable/disable support for QUIC and HTTP/3. Enabled by default.",
+        )
         self.add_option(
             "websocket",
             bool,
@@ -148,13 +163,6 @@ class Options(optmanager.OptManager):
             "Enable/disable raw TCP connections. "
             "TCP connections are enabled by default. ",
         )
-        self.add_option(
-            "rawudp",
-            bool,
-            True,
-            "Enable/disable raw UDP connections. "
-            "UDP connections are enabled by default. ",
-        )
         self.add_option(
             "ssl_insecure",
             bool,
