@@ -1,6 +1,6 @@
 import asyncio
+import contextlib
 import mimetypes
-import os
 import os.path
 import shlex
 import shutil
@@ -8,22 +8,21 @@ import stat
 import subprocess
 import sys
 import tempfile
-import contextlib
 import threading
 from typing import TypeVar
 
+import urwid
 from tornado.platform.asyncio import AddThreadSelectorEventLoop
 
-import urwid
-
 from mitmproxy import addons
-from mitmproxy import master
 from mitmproxy import log
-from mitmproxy.addons import errorcheck, intercept
+from mitmproxy import master
+from mitmproxy import options
+from mitmproxy.addons import errorcheck
 from mitmproxy.addons import eventstore
+from mitmproxy.addons import intercept
 from mitmproxy.addons import readfile
 from mitmproxy.addons import view
-from mitmproxy.contrib.tornado import patch_tornado
 from mitmproxy.tools.console import consoleaddons
 from mitmproxy.tools.console import defaultkeys
 from mitmproxy.tools.console import keymap
@@ -31,12 +30,11 @@ from mitmproxy.tools.console import palettes
 from mitmproxy.tools.console import signals
 from mitmproxy.tools.console import window
 
-
 T = TypeVar("T", str, bytes)
 
 
 class ConsoleMaster(master.Master):
-    def __init__(self, opts):
+    def __init__(self, opts: options.Options) -> None:
         super().__init__(opts)
 
         self.view: view.View = view.View()
@@ -48,8 +46,6 @@ class ConsoleMaster(master.Master):
         defaultkeys.map(self.keymap)
         self.options.errored.connect(self.options_error)
 
-        self.view_stack = []
-
         self.addons.add(*addons.default_addons())
         self.addons.add(
             intercept.Intercept(),
@@ -57,11 +53,11 @@ class ConsoleMaster(master.Master):
             self.events,
             readfile.ReadFile(),
             consoleaddons.ConsoleAddon(self),
-            keymap.KeymapConfig(),
-            errorcheck.ErrorCheck(log_to_stderr=True),
+            keymap.KeymapConfig(self),
+            errorcheck.ErrorCheck(repeat_errors_on_stderr=True),
         )
 
-        self.window = None
+        self.window: window.Window | None = None
 
     def __setattr__(self, name, value):
         super().__setattr__(name, value)
@@ -135,7 +131,7 @@ class ConsoleMaster(master.Master):
         with self.uistopped():
             try:
                 subprocess.call(cmd)
-            except:
+            except Exception:
                 signals.status_message.send(message="Can't start editor: %s" % c)
             else:
                 with open(name, "r" if text else "rb") as f:
@@ -170,7 +166,7 @@ class ConsoleMaster(master.Master):
         with self.uistopped():
             try:
                 subprocess.call(cmd, shell=False)
-            except:
+            except Exception:
                 signals.status_message.send(
                     message="Can't start external viewer: %s" % " ".join(c)
                 )
@@ -219,7 +215,6 @@ class ConsoleMaster(master.Master):
 
         loop = asyncio.get_running_loop()
         if isinstance(loop, getattr(asyncio, "ProactorEventLoop", tuple())):
-            patch_tornado()
             # fix for https://bugs.python.org/issue37373
             loop = AddThreadSelectorEventLoop(loop)  # type: ignore
         self.loop = urwid.MainLoop(
@@ -241,9 +236,11 @@ class ConsoleMaster(master.Master):
         await super().done()
 
     def overlay(self, widget, **kwargs):
+        assert self.window
         self.window.set_overlay(widget, **kwargs)
 
     def switch_view(self, name):
+        assert self.window
         self.window.push(name)
 
     def quit(self, a):
